/* /////////////////////////////////////////////////////////////////////////////
 * File:        dotnetstl/string_accessors.hpp (formerly dotnetstl_string_accessor.h)
 *
 * Purpose:     A useful tool for accessing a String object's content as a c-string.
 *
 * Created:     24th June 2003
 * Updated:     15th October 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file dotnetstl/string_accessor.hpp
///
/// A useful tool for accessing a String object's content as a c-string.

#ifndef DOTNETSTL_INCL_DOTNETSTL_HPP_STRING_ACCESSOR
#define DOTNETSTL_INCL_DOTNETSTL_HPP_STRING_ACCESSOR

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define DOTNETSTL_VER_DOTNETSTL_HPP_STRING_ACCESSOR_MAJOR      3
# define DOTNETSTL_VER_DOTNETSTL_HPP_STRING_ACCESSOR_MINOR      1
# define DOTNETSTL_VER_DOTNETSTL_HPP_STRING_ACCESSOR_REVISION   1
# define DOTNETSTL_VER_DOTNETSTL_HPP_STRING_ACCESSOR_EDIT       31
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef DOTNETSTL_INCL_H_DOTNETSTL
# include <dotnetstl.h>
#endif /* !DOTNETSTL_INCL_H_DOTNETSTL */
#ifndef STLSOFT_INCL_STLSOFT_HPP_SAP_CAST
# include <stlsoft/sap_cast.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_SAP_CAST */
#ifdef STLSOFT_UNITTEST
# include <string.h>
# include <wchar.h>
#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::dotnetstl */
namespace dotnetstl
{
#else
/* Define stlsoft::dotnet_project */

namespace stlsoft
{

namespace dotnetstl_project
{

#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_string String Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating strings

/// \defgroup dotnetstl_string_library String Library (.netSTL)
/// \ingroup dotnetSTL libraries_string
/// \brief This library provides facilities for defining and manipulating strings in .NET
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Template providing scoped C-string access to a String
template <ss_typename_param_k C>
class c_string_accessor
{
public:
    typedef C           char_type;
    typedef C           *pointer;
    typedef C const     *const_pointer;

// Construction
public:
    /// Construct from a System::String
    ///
    /// \param s The String for which C-string access is to be provided. Must not be NULL
    ss_explicit_k c_string_accessor(System::String const *s)
        : m_s(get_cstring_(s))
    {}
    /// Release resources
    ~c_string_accessor()
    {
        System::Runtime::InteropServices::Marshal::FreeHGlobal(m_s);
    }

    c_string_accessor(c_string_accessor &rhs)
        : m_s(rhs.m_s)
    {
        rhs.m_s = 0;
    }

// Accessors
public:
    /// Implicit conversion operator to a C-string
    operator const_pointer() const
    {
        return m_s;
    }

// Implementation
private:
    pointer get_cstring_(System::String const *s);

// Members
private:
    pointer m_s;

// Not to be implemented
private:
    c_string_accessor &operator =(c_string_accessor const &);
};

////////////////////////////////////////////////////////////////////////////////
// Implementation

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION

STLSOFT_TEMPLATE_SPECIALISATION
inline c_string_accessor<ds_char_a_t>::pointer c_string_accessor<ds_char_a_t>::get_cstring_(System::String const *s)
{
    return sap_cast<ds_char_a_t*>(System::Runtime::InteropServices::Marshal::StringToHGlobalAnsi(const_cast<System::String*>(s)).ToPointer());
}

STLSOFT_TEMPLATE_SPECIALISATION
inline c_string_accessor<ds_char_w_t>::pointer c_string_accessor<ds_char_w_t>::get_cstring_(System::String const *s)
{
    return sap_cast<ds_char_w_t*>(System::Runtime::InteropServices::Marshal::StringToHGlobalUni(const_cast<System::String*>(s)).ToPointer());
}

#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST
# include "./unittest/string_accessor_unittest_.h"
#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group dotnetstl_string_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _STLSOFT_NO_NAMESPACE
} // namespace dotnetstl
#else
} // namespace dotnetstl_project
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* DOTNETSTL_INCL_DOTNETSTL_HPP_STRING_ACCESSOR */

/* ////////////////////////////////////////////////////////////////////////// */
