
#if !defined(STLSOFT_INCL_STLSOFT_HPP_TYPE_TRAITS)
# error This file cannot be directly included, and should only be included within stlsoft/type_traits.hpp
#endif /* !STLSOFT_INCL_STLSOFT_HPP_TYPE_TRAITS */

#if !defined(STLSOFT_UNITTEST)
# error This file cannot be included when not unit-testing STLSoft
#endif /* !STLSOFT_UNITTEST */

namespace unittest
{
#if !defined(STLSOFT_COMPILER_IS_WATCOM)

    namespace
    {
        template<   ss_int_t bIsPointer
                ,   ss_int_t bIsReference
                ,   ss_int_t bIsConst
                ,   ss_int_t bIsVolatile
                ,   ss_int_t bIsArrayOrPointer
                ,   ss_typename_param_k T
                >
        void type_traits_type_tester_(T)
        {
            typedef base_type_traits<T>     base_type_traits_t;

            stlsoft_static_assert(base_type_traits_t::is_pointer            ==  bIsPointer);

# if !defined(STLSOFT_COMPILER_IS_BORLAND) || \
     __BORLANDC__ >= 0x0564

            stlsoft_static_assert(base_type_traits_t::is_reference          ==  bIsReference);
            stlsoft_static_assert(base_type_traits_t::is_const              ==  bIsConst);
            stlsoft_static_assert(base_type_traits_t::is_volatile           ==  bIsVolatile);
            stlsoft_static_assert(base_type_traits_t::is_array_or_pointer   ==  bIsArrayOrPointer);

# endif /* compiler */
        }

        template<ss_typename_param_k T>
        ss_bool_t type_traits_type_tester_( unittest_reporter *r
                                        ,   T
                                        ,   ss_int_t bIsPointer
                                        ,   ss_int_t bIsReference
                                        ,   ss_int_t bIsConst
                                        ,   ss_int_t bIsVolatile)
        {
            ss_bool_t                       bSuccess    =   true;

            typedef base_type_traits<T>     base_type_traits_t;

            if(base_type_traits_t::is_pointer       !=  bIsPointer)
            {
                r->report("compile-time detection of is_pointer failed", __LINE__);
                bSuccess = false;
            }
            if(base_type_traits_t::is_reference !=  bIsReference)
            {
                r->report("compile-time detection of is_reference failed", __LINE__);
                bSuccess = false;
            }
            if(base_type_traits_t::is_const     !=  bIsConst)
            {
                r->report("compile-time detection of is_const failed", __LINE__);
                bSuccess = false;
            }
            if(base_type_traits_t::is_volatile  !=  bIsVolatile)
            {
                r->report("compile-time detection of is_volatile failed", __LINE__);
                bSuccess = false;
            }

            return bSuccess;
        }

        ss_bool_t test_stlsoft_type_traits(unittest_reporter *r)
        {
            ss_bool_t               bSuccess    =   true;

            unittest_initialiser    init(r, "STLSoft", "type_traits", __FILE__);

# ifdef __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
            int                     i       =   0;
            const int               ci      =   1;
            volatile int            vi      =   2;
            const volatile int      cvi     =   3;

            int                     *pi     =   &i;
            int const               *pci    =   &ci;
            int volatile            *pvi    =   &vi;
            int const volatile      *pcvi   =   &cvi;

            int                     &ri     =   i;
            int const               &rci    =   ci;
            int volatile            &rvi    =   vi;
            int const volatile      &rcvi   =   cvi;

            int                     *&rpi   =   pi;
            int const               *&rpci  =   pci;
#  if !defined(STLSOFT_COMPILER_IS_COMO)
            int volatile            *&rpvi  =   pvi;
            int const volatile      *&rpcvi =   pcvi;
#  endif /* !STLSOFT_COMPILER_IS_COMO */

#  ifdef __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT
            int                     ai[]    =   { 4 };
            int const               aci[]   =   { 5 };
            int volatile            avi[]   =   { 6 };
            int const volatile      acvi[]  =   { 7 };


            STLSOFT_SUPPRESS_UNUSED(ai);
            STLSOFT_SUPPRESS_UNUSED(aci);
            STLSOFT_SUPPRESS_UNUSED(avi);
            STLSOFT_SUPPRESS_UNUSED(acvi);

#  endif /* __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT */

#if 1
            type_traits_type_tester_<0, 0, 0, 0, 0>(i);
#  if !defined(STLSOFT_COMPILER_IS_DMC)
            type_traits_type_tester_<0, 0, 1, 0, 0, const int>(ci);
            type_traits_type_tester_<0, 0, 0, 1, 0, volatile int>(vi);
            type_traits_type_tester_<0, 0, 1, 1, 0, const volatile int>(cvi);
#  endif /* compiler */
#endif /* 0 */

#if 1
            type_traits_type_tester_<1, 0, 0, 0, 1>(pi);
            type_traits_type_tester_<1, 0, 1, 0, 1>(pci);
            type_traits_type_tester_<1, 0, 0, 1, 1>(pvi);
            type_traits_type_tester_<1, 0, 1, 1, 1>(pcvi);
#endif /* 0 */

#if 1
            type_traits_type_tester_<0, 1, 0, 0, 0, int&>(ri);
            type_traits_type_tester_<0, 1, 1, 0, 0, int const&>(rci);
            type_traits_type_tester_<0, 1, 0, 1, 0, int volatile&>(rvi);
            type_traits_type_tester_<0, 1, 1, 1, 0, int const volatile&>(rcvi);
#endif /* 0 */

#if 1
            type_traits_type_tester_<1, 1, 0, 0, 1, int*&>(rpi);
            type_traits_type_tester_<1, 1, 1, 0, 1, int const*&>(rpci);
#  if !defined(STLSOFT_COMPILER_IS_COMO)
            type_traits_type_tester_<1, 1, 0, 1, 1, int volatile*&>(rpvi);
            type_traits_type_tester_<1, 1, 1, 1, 1, int const volatile*&>(rpcvi);
#  endif /* !STLSOFT_COMPILER_IS_COMO */
#endif /* 0 */

#if 1
#  if defined(__STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT) && \
      !defined(STLSOFT_COMPILER_IS_GCC) && \
      (   !defined(STLSOFT_COMPILER_IS_MWERKS) || \
          (__MWERKS__ & 0xFF00) >= 0x3000)
          
            type_traits_type_tester_<0, 0, 0, 0, 1, int[]>(ai);
            type_traits_type_tester_<0, 0, 1, 0, 1, const int[]>(aci);
            type_traits_type_tester_<0, 0, 0, 1, 1, volatile int[]>(avi);
            type_traits_type_tester_<0, 0, 1, 1, 1, const volatile int[]>(acvi);
#  endif /* __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT */
#endif /* 0 */

#if 1
            stlsoft_static_assert(base_type_traits<int>::is_pointer     ==  0);
            stlsoft_static_assert(base_type_traits<int>::is_reference   ==  0);
            stlsoft_static_assert(base_type_traits<int>::is_const       ==  0);
            stlsoft_static_assert(base_type_traits<int>::is_volatile    ==  0);
#else /* ? 0 */

            bSuccess = type_traits_type_tester_(r, i, 0, 0, 0, 0) && bSuccess;
            bSuccess = type_traits_type_tester_<const int>(r, ci, 0, 0, 1, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, vi, 0, 0, 0, 1) && bSuccess;
            bSuccess = type_traits_type_tester_(r, cvi, 0, 0, 1, 1) && bSuccess;

            bSuccess = type_traits_type_tester_(r, pi, 1, 0, 0, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, pci, 1, 0, 1, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, pvi, 1, 0, 0, 1) && bSuccess;
            bSuccess = type_traits_type_tester_(r, pcvi, 1, 0, 1, 1) && bSuccess;

            bSuccess = type_traits_type_tester_(r, ri, 0, 1, 0, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, rci, 0, 1, 1, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, rvi, 0, 1, 0, 1) && bSuccess;
            bSuccess = type_traits_type_tester_(r, rcvi, 0, 1, 1, 1) && bSuccess;

#  ifdef __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT
            bSuccess = type_traits_type_tester_(r, ai, 0, 0, 0, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, aci, 0, 0, 1, 0) && bSuccess;
            bSuccess = type_traits_type_tester_(r, avi, 0, 0, 0, 1) && bSuccess;
            bSuccess = type_traits_type_tester_(r, acvi, 0, 0, 1, 1) && bSuccess;
#  endif /* __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT */
#endif /* 0 */


# endif /* __STLSOFT_CF_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT */

            return bSuccess;
        }

        unittest_registrar    unittest_stlsoft_type_traits(test_stlsoft_type_traits);
    } // anonymous namespace

#endif /* compiler */

} // namespace unittest
