/* ////////////////////////////////////////////////////////////////////////////
 * File:        winstl_processheap_allocator.h
 *
 * Purpose:     processheap_allocator class.
 *
 * Created:     25th February 2002
 * Updated:     22nd September 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2002-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file winstl_processheap_allocator.h
///
/// processheap_allocator class.

#ifndef WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
#define WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_MAJOR     2
# define WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_MINOR     2
# define WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_REVISION  5
# define WINSTL_VER_H_WINSTL_PROCESSHEAP_ALLOCATOR_EDIT      62
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_H_WINSTL
# include <winstl.h>
#endif /* !WINSTL_INCL_H_WINSTL */
#ifndef STLSOFT_INCL_STLSOFT_HPP_ALLOCATOR_BASE
# include <stlsoft/allocator_base.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_ALLOCATOR_BASE */
#ifndef STLSOFT_INCL_STLSOFT_HPP_SAP_CAST
# include <stlsoft/sap_cast.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_SAP_CAST */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_allocator Allocator Library
/// \ingroup libraries
/// \brief This library provides STL-compatible <code><b>allocator</b></code> types

/// \weakgroup winstl_allocator_library Allocator Library (WinSTL)
/// \ingroup WinSTL libraries_allocator
/// \brief This library provides STL-compatible <code><b>allocator</b></code> types for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ss_typename_param_k T>
class processheap_allocator;

// Specialisation for void
STLSOFT_TEMPLATE_SPECIALISATION
class processheap_allocator<void>
{
public:
    typedef void                                value_type;
    typedef processheap_allocator<void>         class_type;
    typedef void                                *pointer;
    typedef void const                          *const_pointer;
    typedef ptrdiff_t                           difference_type;
    typedef ws_size_t                           size_type;

#ifdef STLSOFT_LF_ALLOCATOR_REBIND_SUPPORT
    /// The allocator <b><code>rebind</code></b> structure
    template <ss_typename_param_k U>
    struct rebind
    {
        typedef processheap_allocator<U>        other;
    };
#endif /* STLSOFT_LF_ALLOCATOR_REBIND_SUPPORT */
};

#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/// STL Allocator based on the Win32 process heap
///
/// \param T The value_type of the allocator
template <ss_typename_param_k T>
class processheap_allocator
{
public:
    /// The value type
    typedef T                                   value_type;
    /// The type of the current parameterisation
    typedef processheap_allocator<value_type>   class_type;
    /// The pointer type
    typedef value_type                          *pointer;
    /// The non-mutating (const) pointer type
    typedef value_type const                    *const_pointer;
    /// The reference type
    typedef value_type                          &reference;
    /// The non-mutating (const) reference type
    typedef value_type const                    &const_reference;
    /// The difference type
    typedef ptrdiff_t                           difference_type;
    /// The size type
    typedef ws_size_t                           size_type;
    /// The type used in deallocate()
#ifdef STLSOFT_CF_ALLOCATOR_TYPED_DEALLOCATE_POINTER
    typedef pointer                             deallocate_pointer;
#else /* ? STLSOFT_CF_ALLOCATOR_TYPED_DEALLOCATE_POINTER */
    typedef void                                *deallocate_pointer;
#endif /* STLSOFT_CF_ALLOCATOR_TYPED_DEALLOCATE_POINTER */

#ifdef STLSOFT_LF_ALLOCATOR_REBIND_SUPPORT
    /// The allocator <b><code>rebind</code></b> structure
    template <ss_typename_param_k U>
    struct rebind
    {
        typedef processheap_allocator<U>         other;
    };
#endif /* STLSOFT_LF_ALLOCATOR_REBIND_SUPPORT */

// Construction
public:
    /// Default constructor
    processheap_allocator() stlsoft_throw_0()
        : m_processheap(::GetProcessHeap())
    {}
    /// Copy constructor
#ifndef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    processheap_allocator(processheap_allocator const &/* rhs */) stlsoft_throw_0()
        : m_processheap(::GetProcessHeap())
    {}
#else /* ? __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
    template <ss_typename_param_k U>
    processheap_allocator(processheap_allocator<U> const &/* rhs */) stlsoft_throw_0()
        : m_processheap(::GetProcessHeap())
    {}
#endif /* !__STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
    /// Destructor
    ~processheap_allocator() stlsoft_throw_0()
    {}

    /// Copy assignment operator
    ///
    /// \note This has to be provided, to avoid precipitating C4217 with Visual C++
    class_type &operator =(class_type const &rhs)
    {
        return *this;
    }

// Attributes
public:
    /// Returns the maximum number of allocatable entities
    size_type max_size() const stlsoft_throw_0()
    {
        return static_cast<size_type>(-1) / sizeof(value_type);
    }

// Conversion
public:
    /// Returns the address corresponding to the given reference
    ///
    /// \param x A reference to a \c value_type instance whose address will be calculated
    pointer address(reference x) const stlsoft_throw_0()
    {
        return &x;
    }
    /// Returns the address corresponding to the given non-mutable (const) reference
    ///
    /// \param x A non-mutable (const) reference to a \c value_type instance whose address will be calculated
    const_pointer address(const_reference x) const stlsoft_throw_0()
    {
        return &x;
    }

// Allocation
public:
    /// Allocates a block of memory sufficient to store \c n elements of type \c value_type
    ///
    /// \param n The number of elements to allocate
    /// \param pv A hint to enhance localisation (not used in this class)
    /// \return The allocated block, or the null pointer (if the allocation fails and the translation does not support throwing exceptions upon memory exhaustion)
    pointer allocate(size_type n, processheap_allocator<void>::const_pointer pv = NULL)
    {
        STLSOFT_SUPPRESS_UNUSED(pv);

        processheap_allocator<void>::pointer p   =   static_cast<processheap_allocator<void>::pointer>(::HeapAlloc(m_processheap, 0, n * sizeof(value_type)));

        if(p == NULL)
        {
#ifdef __STLSOFT_CF_THROW_BAD_ALLOC
            throw ::std::bad_alloc();
#endif /* __STLSOFT_CF_THROW_BAD_ALLOC */
        }
        else
        {
            ws_size_t   size    =   static_cast<ws_size_t>(::HeapSize(m_processheap, 0, p));

            winstl_assert(size >= n * sizeof(value_type));
            STLSOFT_SUPPRESS_UNUSED(size);
        }

        return static_cast<pointer>(p);
    }

#ifdef STLSOFT_CF_ALLOCATOR_CHARALLOC_METHOD
    /// The type used in deallocate
    char *_Charalloc(size_type n)
    {
        return sap_cast<char*>(allocate(n, NULL));
    }
#endif /* STLSOFT_CF_ALLOCATOR_CHARALLOC_METHOD */

    /// Deallocates a pointer
    ///
    /// \param p The memory block to deallocate
    /// \param n The number of blocks to deallocate
    void deallocate(pointer p, size_type n)
    {
        STLSOFT_SUPPRESS_UNUSED(n);

        ::HeapFree(m_processheap, 0, p);
    }

    /// Deallocates a pointer
    ///
    /// \param p The memory block to deallocate
    void deallocate(pointer p)
    {
        ::HeapFree(m_processheap, 0, p);
    }

// Operations
public:
    /// In-place constructs an instance of the \c value_type, with the given value
    ///
    /// \param p The location in which to construct the instance
    /// \param x The value with which to copy construct the instance
    void construct(pointer p, value_type const &x)
    {
        winstl_assert(NULL != p);

        new(p) value_type(x);
    }

    /// In-place constructs an instance of the \c value_type
    ///
    /// \param p The location in which to construct the instance
    void construct(pointer p)
    {
        winstl_assert(NULL != p);

        new(p) value_type();
    }

    /// In-place destroys an instance
    ///
    /// \param p The instance whose destructor is to be called
    void destroy(pointer p) stlsoft_throw_0()
    {
        winstl_assert(NULL != p);

        winstl_destroy_instance(T, value_type, p);
    }

// Members
protected:
    HANDLE  m_processheap;

// Not to be implemented
private:
//    class_type const &operator =(class_type const &rhs);
};

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ss_typename_param_k T>
inline ws_bool_t operator ==(processheap_allocator<T> const &/* lhs */, processheap_allocator<T> const &/* rhs */)
{
    return ws_true_v;
}

template <ss_typename_param_k T>
inline ws_bool_t operator !=(processheap_allocator<T> const &/* lhs */, processheap_allocator<T> const &/* rhs */)
{
    return ws_false_v;
}

#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    namespace
    {
        ss_bool_t test_winstl_processheap_allocator(unittest_reporter *r)
        {
            using stlsoft::unittest::unittest_initialiser;

            ss_bool_t               bSuccess    =   true;

            unittest_initialiser    init(r, "WinSTL", "processheap_allocator", __FILE__);

            typedef processheap_allocator<int>  int_allocator_t;

            int_allocator_t ator1;

            int     *pi1    =   ator1.allocate(100);

            if(NULL != pi1)
            {
                ator1.construct(pi1, 1968);

                if(1968 != *pi1)
                {
                    r->report("construct() failed", __LINE__);
                    bSuccess = false;
                }
            }

            ator1.deallocate(pi1);

            return bSuccess;
        }

        unittest_registrar    unittest_winstl_processheap_allocator(test_winstl_processheap_allocator);
    } // anonymous namespace

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_allocator_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */

/* ////////////////////////////////////////////////////////////////////////// */
