/* /////////////////////////////////////////////////////////////////////////////
 * File:        x/atlstl/about_dialog.hpp
 *
 * Purpose:     Enhancement of ATLSTL's AboutDialog, which replaces hyperlinking
 *              buttons with Synesis's hyperlink control, and fixes up version
 *              information with static controls containing version strings.
 *
 * Created:     28th December 2004
 * Updated:     8th October 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2004-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file x/atlstl/about_dialog.hpp
///
/// Enhancement of ATLSTL's AboutDialog, which replaces hyperlinking buttons
/// with Synesis's hyperlink control, and fixes up version information with
/// static controls containing version strings.

#ifndef ATLSTL_INCL_X_ATLSTL_HPP_ABOUT_DIALOG
#define ATLSTL_INCL_X_ATLSTL_HPP_ABOUT_DIALOG

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define ATLSTL_VER_X_ATLSTL_HPP_ABOUT_DIALOG_MAJOR     1
# define ATLSTL_VER_X_ATLSTL_HPP_ABOUT_DIALOG_MINOR     2
# define ATLSTL_VER_X_ATLSTL_HPP_ABOUT_DIALOG_REVISION  2
# define ATLSTL_VER_X_ATLSTL_HPP_ABOUT_DIALOG_EDIT      10
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef ATLSTL_INCL_H_ATLSTL
# include <atlstl.h>
#endif /* !ATLSTL_INCL_H_ATLSTL */
#ifndef ATLSTL_INCL_ATLSTL_HPP_ABOUT_DIALOG
# include <atlstl/about_dialog.hpp>     // Include the ATLSTL AboutDialog header
#endif /* !ATLSTL_INCL_ATLSTL_HPP_ABOUT_DIALOG */
#ifndef STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER
# include <stlsoft/auto_buffer.hpp>       // auto_buffer
#endif /* !STLSOFT_INCL_STLSOFT_HPP_AUTO_BUFFER */
#ifndef STLSOFT_INCL_STLSOFT_HPP_MALLOC_ALLOCATOR
# include <stlsoft/malloc_allocator.hpp>  // malloc_allocator (ATL uses malloc() / free())
#endif /* !STLSOFT_INCL_STLSOFT_HPP_MALLOC_ALLOCATOR */
#ifndef WINSTL_INCL_WINSTL_HPP_STRING_ACCESS
# include <winstl/string_access.hpp>
#endif /* !WINSTL_INCL_WINSTL_HPP_STRING_ACCESS */
#include <MCHprlnk.h>                   // Synesis' Hyperlink control
#include <MWUser.h>                     // Synesis' ReplaceChildWindow function
#include <MWVerInf.h>                   // Synesis' Version info

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _ATLSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
/* There is no stlsoft namespace, so must define ::atlstl */
namespace atlstl
{
# else
/* Define stlsoft::atlstl_project */

namespace stlsoft
{

namespace atlstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// About dialog class based on a type that specifies an Id, and a parent window
/// type
template<   class   T
        ,   class   B = CWindow
        >
class SyAboutDialog
    : public AboutDialog<T, B>
{
public:
    typedef AboutDialog<T, B>               parent_class_type;
    typedef SyAboutDialog<T, B>             class_type;
private:
    typedef ::stlsoft::auto_buffer< TCHAR
                                ,   ::stlsoft::malloc_allocator<TCHAR>
                                ,   512
                                >           buffer_type;

public:
    BEGIN_MSG_MAP(class_type)
        MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
        COMMAND_RANGE_HANDLER(IDOK, IDNO, OnCloseCmd)
        CHAIN_MSG_MAP(parent_class_type)
    END_MSG_MAP()

protected:
    LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL &bHandled)
    {
        bHandled = false;

        // 1. Replace all buttons which have hyperlink URLs
        ReplaceButtonsWithHyperlinks();

        // 2. Detect a 'version' static, and update the contents
        InstantiateVersionLabel();

        return TRUE;
    }

    LRESULT OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
    {
        EndDialog(static_cast<int>(wID));

        return 0;
    }

public:
    BOOL EndDialog(int nRetCode)
    {
        ATLASSERT(::IsWindow(m_hWnd));
        return ::EndDialog(m_hWnd, nRetCode);
    }

private:
    void ReplaceButtonsWithHyperlinks()
    {
        ::EnumChildWindows(*this, ReplaceProc, (LPARAM)this);
    }

    static BOOL CALLBACK ReplaceProc(HWND hwndChild, LPARAM lParam)
    {
        class_type  *this_  =   (class_type*)lParam;

        this_->ReplaceChild(hwndChild);

        return TRUE;
    }

    void ReplaceChild(HWND hwndChild)
    {
        buffer_type buffer(1 + ::GetWindowTextLength(hwndChild));
        const int   len =   ::GetWindowText(hwndChild, &buffer[0], buffer.size());

        buffer[len] = '\0';

        if(this->CanChangeToHyperlink(hwndChild, &buffer[0], len))
        {
            HWND    hwndNew =   ::SynesisWin::ReplaceChildWindowStyle(  hwndChild
                                                                    ,   _T("SyHyperlink32")
                                                                    ,   NULL    /* Use existing caption */
                                                                    ,   -1, -1, -1, -1
                                                                    ,   MHLS_LEFT | MHLS_CENTREIMAGE
                                                                    ,   MHLS_JUSTIFYMASK
                                                                    ,   0
                                                                    ,   0);
            if(hwndNew != NULL)
            {
                MHyperlink_SetBackClr(hwndNew, CLR_INVALID);
            }
        }
    }

    void InstantiateVersionLabel()
    {
        HWND    hwndVersion =   NULL;

        ::EnumChildWindows(*this, FindVersionLabelProc, (LPARAM)&hwndVersion);

        if( NULL != hwndVersion &&
            ::SynesisWin::WinVer_Init() >= 0)
        {
            ::SynesisWin::LPWinVerInfo  verInfo =   ::SynesisWin::WinVer_GetVersionInformation(NULL);

            if(NULL != verInfo)
            {
                TCHAR   fmt[512];
                TCHAR   sz[768];

                _tcscpy(&sz[0], ::stlsoft::c_str_ptr(hwndVersion));

                sz[stlsoft_num_elements(sz) - 1] = '\0';

                LPCTSTR     begin   =   &sz[0];
                LPCTSTR     end     =   &sz[0] + _tcslen(sz);
                LPTSTR      dest    =   &fmt[0];
                const TCHAR DATE[]  =   __DATE__ _T(" ") __TIME__;

                for(fmt[0] = '\0'; begin != end; )
                {
                    LPCTSTR p = _tcsstr(begin, _T("%B"));

                    if(NULL == p)
                    {
                        _tcsncpy(dest, begin, end - begin);

                        break;
                    }
                    else
                    {
                        _tcsncpy(dest, begin, p - begin);
                        dest += p - begin;
                        _tcscpy(dest, DATE);
                        dest += stlsoft_num_elements(DATE) - 1;

                        begin = p + 2;

                        ATLASSERT(begin <= end);
                    }

                }

                ::SynesisWin::WinVer_sprintf(verInfo, &sz[0], stlsoft_num_elements(sz), fmt);

                ::SetWindowText(hwndVersion, &sz[0]);

                ::SynesisWin::WinVer_CloseVersionInformationA(verInfo);
            }

            ::SynesisWin::WinVer_Uninit();
        }
    }

    static BOOL CALLBACK FindVersionLabelProc(HWND hwndChild, LPARAM lParam)
    {
        buffer_type buffer(1 + ::GetWindowTextLength(hwndChild));
        const int   len =   ::GetWindowText(hwndChild, &buffer[0], buffer.size());

        buffer[len] = '\0';

        if(NULL != _tcsstr(&buffer[0], "%j.%n"))
        {
            *(HWND*)lParam = hwndChild;

            return false;
        }

        return TRUE;
    }

private:
    virtual BOOL CanChangeToHyperlink(HWND hwndChild, LPCTSTR windowText, int windowTextLen)
    {
        const TCHAR     http[]      =   "http://";
        const LRESULT   ctrlCode    =   ::SendMessage(hwndChild, WM_GETDLGCODE, 0, 0L);

        if(0 == (DLGC_BUTTON & ctrlCode))
        {
            return false;
        }

        if(windowTextLen < stlsoft_num_elements(http))
        {
            return false;
        }

        if(0 != _tcsnicmp(&http[0], windowText, stlsoft_num_elements(http) - 1))
        {
            return false;
        }

        return true;
    }
};

/// About dialog class based from an Id
template<   UINT    ID
        ,   class   B = CWindow
        >
class SyAboutDialogId
    : public SyAboutDialog<IDD_to_type<ID>, B>
{
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _ATLSTL_NO_NAMESPACE
# if defined(_STLSOFT_NO_NAMESPACE) || \
     defined(STLSOFT_DOCUMENTATION_SKIP_SECTION)
} // namespace atlstl
# else
} // namespace atlstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_ATLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* ATLSTL_INCL_X_ATLSTL_HPP_ABOUT_DIALOG */

/* ////////////////////////////////////////////////////////////////////////// */
