/* /////////////////////////////////////////////////////////////////////////////
 * File:        x/wtlstl_x_simple_help_window.h (formerly MCHlpWnd.h, ::SynesisCstCtrls)
 *
 * Purpose:     Definition of the SimpleHelpWindow class.
 *
 * Created:     1st November 1999
 * Updated:     17th July 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 1999-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file x/wtlstl_x_simple_help_window.h     Definition of the SimpleHelpWindow class

#ifndef WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW
#define WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW_MAJOR    1
# define WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW_MINOR    1
# define WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW_REVISION 3
# define WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW_EDIT     16
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WTLSTL_INCL_H_WTLSTL
# include <wtlstl.h>
#endif /* !WTLSTL_INCL_H_WTLSTL */
#include <MCHlpWnd.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WTLSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::wtlstl */
namespace wtlstl
{
# else
/* Define stlsoft::wtlstl_project */

namespace stlsoft
{

namespace wtlstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WTLSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// A bolt-in that automatically handles context-sensitive help
///
/// \param W The window class, e.g. MyDialog, as in SimpleHelpWindow<MyDialog, true>
/// \param USE_DEFAULT_TEXT If true it
template<   typename    W
        ,   bool        USE_DEFAULT_TEXT = false
        >
class SimpleHelpWindow
{
public:
    /// The type of the bolted window class
    typedef W                                       window_type;
    /// The current parameterisation of the type
    typedef SimpleHelpWindow<W, USE_DEFAULT_TEXT>    class_type;

/// Message map
public:
    BEGIN_MSG_MAP(SimpleHelpWindow)
        MESSAGE_HANDLER(WM_HELP, OnHelp)
    END_MSG_MAP()

/// Message handlers
private:
    static HWND AttemptFindChildById(HWND hwnd, int id)
    {
#if (   defined(_SYNSOFT_INCL_H_MWUSER) || \
        defined(__SYNSOFT_INCL_H_MWUSER)) && \
    (   _SYNSOFT_VER_H_MWUSER_MAJOR > 2 || \
        (   _SYNSOFT_VER_H_MWUSER_MAJOR == 2 && \
            _SYNSOFT_VER_H_MWUSER_MINOR >= 11))
        return __SYNSOFT_GEN_NS_QUAL(SynesisWin, FindChildById)(hwnd, id);
#else /* ? _SYNSOFT_INCL_H_MWUSER || __SYNSOFT_INCL_H_MWUSER*/
        HWND    hwndChild = ::GetDlgItem(hwnd, id);

        return hwndChild != NULL ? hwndChild : hwnd;
#endif /* _SYNSOFT_INCL_H_MWUSER || __SYNSOFT_INCL_H_MWUSER*/
    }

    LRESULT OnHelp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL &bHandled)
    {
        LPHELPINFO  lphi    =   (LPHELPINFO)lParam;

        if(lphi->iContextType == HELPINFO_WINDOW)
        {
            HWND    hwndHelp    =   (HWND)lphi->hItemHandle;
            int     id          =   lphi->iCtrlId;
            RECT    rc;
            TCHAR   szHelp[256];
            LPCTSTR pcszHelp    =   szHelp;
            LPCTSTR helpDefaults[2] =
            {
                    NULL
                ,   _T("Help is not available for this item")
            };

            if(::LoadString(_Module.GetResourceInstance(), id, szHelp, NUM_ELEMENTS(szHelp)) == 0)
            {
                pcszHelp = helpDefaults[USE_DEFAULT_TEXT];
            }

            if(pcszHelp != NULL)
            {
                hwndHelp = AttemptFindChildById(hwndHelp, id);

                ::GetWindowRect(hwndHelp, &rc);
                ::OffsetRect(&rc, 10 + (rc.right - rc.left) / 2, 10 + (rc.bottom - rc.top) / 2);
                __SYNSOFT_GEN_NS_QUAL(SynesisCstCtrls, CreateHelpWindow)(hwndHelp, (_mPC(POINT))&rc, pcszHelp, 0);
            }
        }

        bHandled = true;

        return 0;
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WTLSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
} // namespace wtlstl
# else
} // namespace wtlstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WTLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !WTLSTL_INCL_X_H_WTLSTL_SIMPLE_HELP_WINDOW */

/* ////////////////////////////////////////////////////////////////////////// */
