/* /////////////////////////////////////////////////////////////////////////////
 * File:        x/wtlstl_x_synesis_controls.h
 *
 * Purpose:     Definition of the SimpleHelpWindow class.
 *
 * Created:     5th May 2003
 * Updated:     27th July 2005
 *
 * Home:        http://stlsoft.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/// \file x/wtlstl_x_synesis_controls.h

#ifndef WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS
#define WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS

#ifndef STLSOFT_DOCUMENTATION_SKIP_SECTION
# define WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS_MAJOR      1
# define WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS_MINOR      6
# define WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS_REVISION   1
# define WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS_EDIT       25
#endif /* !STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WTLSTL_INCL_H_WTLSTL
# include <wtlstl.h>
#endif /* !WTLSTL_INCL_H_WTLSTL */
#ifndef STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS
# include <stlsoft/string_access.hpp>
#endif /* !STLSOFT_INCL_STLSOFT_HPP_STRING_ACCESS */
#ifndef STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
# include "stlsoft_auto_buffer.h"       // stlsoft::auto_buffer
#endif /* !STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef STLSOFT_INCL_H_STLSOFT_MALLOC_ALLOCATOR
# include "stlsoft_malloc_allocator.h"  // stlsoft::malloc_allocator
#endif /* !STLSOFT_INCL_H_STLSOFT_MALLOC_ALLOCATOR */
#include <MCEdtBar.h>                   // Synesis Software EditBar custom control
#include <MCHexVw.h>                    // Synesis Software HexView custom control
#include <MCLstBox.h>                   // Synesis Software ListBox custom control

#include <stlsoft/unary_function_output_iterator_adaptor.hpp>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WTLSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::wtlstl */
namespace wtlstl
{
# else
/* Define stlsoft::wtlstl_project */

namespace stlsoft
{

namespace wtlstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WTLSTL_NO_NAMESPACE */

using ::SynesisCstCtrls::CellItemA;
using ::SynesisCstCtrls::CellItemW;
using ::SynesisCstCtrls::CellItem;
using ::SynesisCstCtrls::ColumnItem;
using ::SynesisCstCtrls::FindInfo;
using ::SynesisCstCtrls::PFnCellComparator;

using ::SynesisCstCtrls::HexViewBorders;
using ::SynesisCstCtrls::LPHexViewBorders;
using ::SynesisCstCtrls::LPCHexViewBorders;
using ::SynesisCstCtrls::HexViewData;
using ::SynesisCstCtrls::LPHexViewData;
using ::SynesisCstCtrls::LPCHexViewData;

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Adaptor class template to implement facades for the Synesis Software Custom Controls
template <class TControl>
class SynesisControlCommonFeatures
{
    typedef TControl    control_type;

public:
    // Colours.
    COLORREF SetBkClr(COLORREF clrBk)
    {
        return MControl_SetBkClr(get_hwnd_(), clrBk);
    }

    COLORREF GetBkClr() const
    {
        return MControl_GetBkClr(get_hwnd_());
    }

    COLORREF SetFocusClr(COLORREF clrFocus)
    {
        return MControl_SetFocusClr(get_hwnd_(), clrFocus);
    }

    COLORREF GetFocusClr() const
    {
        ATLASSERT(get_hwnd_());

        return MControl_GetFocusClr(get_hwnd_());
    }

    // Colours.
    COLORREF SetTextClr(COLORREF clrTxt, int nItem)
    {
        return MControl_SetTextClr(get_hwnd_(), nItem, clrTxt);
    }

    COLORREF GetTextClr(int nItem) const
    {
        return MControl_GetTextClr(get_hwnd_(), nItem);
    }

    COLORREF SetTextBkClr(COLORREF clrTxtBk, int nItem)
    {
        return MControl_SetTextBkClr(get_hwnd_(), nItem, clrTxtBk);
    }

    COLORREF GetTextBkClr(int nItem) const
    {
        return MControl_GetTextBkClr(get_hwnd_(), nItem);
    }

    // Image lists
    HIMAGELIST SetImageList(HIMAGELIST hil, UINT u)
    {
        return MControl_SetImageList(get_hwnd_(), hil, u);
    }
    HIMAGELIST GetImageList(UINT u, UINT *pu = NULL)
    {
        return MControl_GetImageList(get_hwnd_(), u, pu);
    }

    // Selections.
    int DeleteAllSelectedItems()
    {
        return MControl_DeleteAllSelectedItems(get_hwnd_());
    }

    int GetSelectedCount() const
    {
        return MControl_GetSelectedCount(get_hwnd_());
    }

    int GetOnlySelection() const
    {
        return MControl_GetOnlySel(get_hwnd_());
    }

    // Item reactangle
    BOOL GetItemRect(int nItem, LPRECT lprc) const
    {
        return MControl_GetItemRect(get_hwnd_(), nItem, lprc);
    }

    BOOL GetTextRect(int nItem, LPRECT lprc) const
    {
        return MControl_GetTextRect(get_hwnd_(), nItem, lprc);
    }

    BOOL GetImageRect(int nItem, LPRECT lprc) const
    {
        return MControl_GetImageRect(get_hwnd_(), nItem, lprc);
    }

    // Moving
    int MoveItemUp(int nItem)
    {
        return MControl_MoveItemUp(get_hwnd_(), nItem);
    }

    int MoveItemDown(int nItem)
    {
        return MControl_MoveItemDown(get_hwnd_(), nItem);
    }

    // Error focus.
    BOOL SupportFocusStyle() const
    {
        return MControl_SupportFocusStyle(get_hwnd_());
    }

    int SetFocusStyle(int nStyle)
    {
        return MControl_SetFocusStyle(get_hwnd_(), nStyle);
    }

    int GetFocusStyle() const
    {
        return MControl_GetFocusStyle(get_hwnd_());
    }

    void ErrorFocus()
    {
        MControl_ErrorFocus(get_hwnd_());
    }

    // Sorting
    int SortItems(BOOL bForward)
    {
        return MControl_SortItems(get_hwnd_(), !bForward);
    }

    // Clipboard
    int CopyToClipboard(int nItem, TCHAR cSep) const
    {
        return MControl_CopyToClipboard(get_hwnd_(), cSep, nItem);
    }

# ifdef __IDataObject_FWD_DEFINED__
    int GetAsIDataObject(LPDATAOBJECT FAR *ppDataObj, int nItem, TCHAR cSep) const
    {
        return MControl_GetAsIDataObject(get_hwnd_(), nItem, cSep, ppDataObj);
    }
# endif // __IDataObject_FWD_DEFINED__

# ifdef WIN32
    BOOL SetUnicodeFormat(BOOL bUnicode)
    {
        return MControl_SetUnicodeFormat(get_hwnd_(), bUnicode);
    }

    BOOL GetUnicodeFormat() const
    {
        return MControl_GetUnicodeFormat(get_hwnd_());
    }
# endif // WIN32

/// Implementation
private:
    HWND    get_hwnd_()
    {
        ATLASSERT(::IsWindow(static_cast<control_type*>(this)->m_hWnd));

        return static_cast<control_type*>(this)->m_hWnd;
    }
    HWND    get_hwnd_() const
    {
        ATLASSERT(::IsWindow(static_cast<control_type const*>(this)->m_hWnd));

        return static_cast<control_type const*>(this)->m_hWnd;
    }
};

/* /////////////////////////////////////////////////////////////////////////////
 * CSyListBoxT
 */

/// Class facade for the Synesis Software Listbox Custom Control
template <class TBase>
class CSyListBoxT
    : public CListBoxT<TBase>
    , public SynesisControlCommonFeatures<CSyListBoxT<TBase> >
{
private:
    typedef ::stlsoft::auto_buffer<TCHAR, ::stlsoft::malloc_allocator<TCHAR>, 256>  buffer_type;
public:
    typedef CListBoxT<TBase>                                                        parent_class_type;
    typedef CSyListBoxT<TBase>                                                      class_type;

/// \name Construction
/// @{
public:
    CSyListBoxT(HWND hWnd = NULL)
        : parent_class_type(hWnd)
    {}

    class_type &operator =(HWND hWnd)
    {
        m_hWnd = hWnd;
        return *this;
    }
/// @}

/// \name Attributes
/// @{
public:
    static LPCTSTR GetWndClassName()
    {
        return _T("SyListBox32");
    }
/// @}

// Column/tab-stop attributes
    int SetColumnCount(int nCols, UINT nFlags = MLBCN_MINIMUM)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetColumnCount(this->m_hWnd, nCols, nFlags);
    }

    int GetColumnCount() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetColumnCount(this->m_hWnd, NULL);
    }

    int SetColumnWidth(int nCol, int cxCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetColumnWidth(this->m_hWnd, nCol, cxCol);
    }

    int GetColumnWidth(int nCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetColumnWidth(this->m_hWnd, nCol);
    }

    // Edit cell
    int SetEditStyle(int nStyle)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetEditCellStyle(this->m_hWnd, nStyle);
    }

    int GetEditStyle() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetEditCellStyle(this->m_hWnd);
    }

    int EditCell(int nItem, int nCol = 0)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_EditCell(this->m_hWnd, nItem, nCol);
    }

    int CommitEditCell()
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_CommitEditCell(this->m_hWnd);
    }

    int CancelEditCell()
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_CancelEditCell(this->m_hWnd);
    }

    HWND GetEditCell() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetEditCell(this->m_hWnd);
    }

    BOOL GetEditCellAxes(int &nItem, int &nCol) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        nItem=MListBox_GetEditCellAxis(this->m_hWnd, TRUE);
        nCol=MListBox_GetEditCellAxis(this->m_hWnd, FALSE);

        return (nItem>=0 && nCol>=0);
    }

    // Sorting
    int SortItems(PFnCellComparator pfnComp, BOOL bForward = true)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SortItems(this->m_hWnd, -1, !bForward, pfnComp);
    }

    int SortItems(PFnCellComparator pfnComp, UINT nCol, BOOL bForward)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SortItems(this->m_hWnd, nCol, !bForward, pfnComp);
    }

    int SortItems(UINT nCol, BOOL bForward)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SortItems(this->m_hWnd, nCol, !bForward, NULL);
    }

# if _MSC_VER<0x0400
    int SortItems(BOOL bForward)
    {
        return ParentClass::SortItems(bForward);
    }
# endif // _MSC_VER>=0x0400

    // Columns
    int SetColumn(int nCol, ColumnItem FAR *pCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetColumn(this->m_hWnd, nCol, pCol);
    }

    int GetColumn(int nCol, ColumnItem FAR *pCol) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetColumn(this->m_hWnd, nCol, pCol);
    }

    int SetActiveColumn(int nCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetActiveColumn(this->m_hWnd, nCol);
    }

    int GetActiveColumn() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetActiveColumn(this->m_hWnd);
    }

    int AddString(LPCTSTR lpszItem)
    {
        return this->parent_class_type::AddString(lpszItem);
    }
    template <ss_typename_param_k S>
    int AddString(S const &lpszItem)
    {
        return this->AddString(::stlsoft::c_str_ptr(lpszItem));
    }


    // Items.
    int InsertItem(const CellItem FAR *pCell)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_InsertItem(this->m_hWnd, pCell);
    }
    int InsertItem(int nItem, LPCTSTR lpszItem)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<TCHAR *>(lpszItem);
        item.nItem      =   nItem;
        item.nSubItem   =   0;

        return InsertItem(&item);
    }
    template <ss_typename_param_k S>
    int InsertItem(int nItem, S const &lpszItem)
    {
        return this->InsertItem(nItem, ::stlsoft::c_str_ptr(lpszItem));
    }
    int InsertItem(int nItem, int nSubItem, LPCTSTR lpszItem)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<TCHAR *>(lpszItem);
        item.nItem      =   nItem;
        item.nSubItem   =   nSubItem;


        return InsertItem(&item);
    }
    template <ss_typename_param_k S>
    int InsertItem(int nItem, int nSubItem, S const &lpszItem)
    {
        return this->InsertItem(nItem, nSubItem, ::stlsoft::c_str_ptr(lpszItem));
    }
    int InsertItem(int nItem, LPARAM lParam)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE | MCCIF_PARAM;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   LPSTR_TEXTCALLBACK;
        item.nItem      =   nItem;
        item.nSubItem   =   0;
        item.lParam     =   lParam;

        return InsertItem(&item);
    }
    int InsertItem(int nItem, LPCTSTR lpszItem, int nImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE | MCCIF_IMAGE;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<TCHAR *>(lpszItem);
        item.nItem      =   nItem;
        item.nSubItem   =   0;
        item.image      =   nImage;

        return InsertItem(&item);
    }
    template <ss_typename_param_k S>
    int InsertItem(int nItem, S const &lpszItem, int nImage)
    {
        return this->InsertItem(nItem, ::stlsoft::c_str_ptr(lpszItem), nImage);
    }

    BOOL SetItem(const CellItem FAR *pcCell)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetItem(this->m_hWnd, pcCell);
    }
    int SetItem(int nItem, int nSubItem, LPCTSTR lpszItem)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<TCHAR *>(lpszItem);
        item.nItem      =   nItem;
        item.nSubItem   =   nSubItem;

        return SetItem(&item);
    }
    template <ss_typename_param_k S>
    int SetItem(int nItem, int nSubItem, S const &lpszItem)
    {
        return this->SetItem(nItem, nSubItem, ::stlsoft::c_str_ptr(lpszItem));
    }
    int SetItem(int nItem, LPCTSTR lpszItem, int nImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE | MCCIF_IMAGE;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<TCHAR *>(lpszItem);
        item.nItem      =   nItem;
        item.nSubItem   =   0;
        item.image      =   nImage;

        return SetItem(&item);
    }
    template <ss_typename_param_k S>
    int SetItem(int nItem, S const &lpszItem, int nImage)
    {
        return this->SetItem(nItem, ::stlsoft::c_str_ptr(lpszItem), nImage);
    }

    BOOL GetItem(CellItem FAR *pCell) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetItem(this->m_hWnd, pCell);
    }

    int GetItemText(int nItem, int nSubItem, LPTSTR lpszText, int nLen) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        CellItem    item;

        item.mask       =   MCCIF_TYPE | MCCIF_BUFFER;
        item.nItem      =   nItem;
        item.nSubItem   =   nSubItem;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   lpszText;
        item.cchTextMax =   nLen;

        if(!GetItem(&item))
        {
            return 0;
        }
        else
        {
            return item.cchTextMax;
        }
    }

    int SetItemImage(int nItem, int nImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetItemImage(this->m_hWnd, nItem, nImage);
    }

    int GetItemImage(int nItem) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetItemImage(this->m_hWnd, nItem);
    }

    DWORD SetItemState(int nItem, UINT state, UINT stateMask)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetItemState(this->m_hWnd, nItem, stateMask, state);
    }

    DWORD GetItemState(int nItem, UINT stateMask) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetItemState(this->m_hWnd, nItem, stateMask);
    }

    int GetText(int index, LPTSTR buffer) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return parent_class_type::GetText(index, buffer);
    }

#if defined(_WTL_USE_CSTRING) || defined(__ATLSTR_H__)
    int GetText(int index, CString& strText) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return parent_class_type::GetText(index, strText);
    }
#endif //defined(_WTL_USE_CSTRING) || defined(__ATLSTR_H__)

    template <ss_typename_param_k S>
    int GetText(int index, S &text) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        int cch =   this->GetTextLen(index);

        if(cch >= 0)
        {
            buffer_type buffer(1 + cch);

            if(buffer.empty())
            {
                cch = -1;
            }
            else
            {
                int cch2    =   this->GetText(index, &buffer[0]);

                if(cch2 >= 0)
                {
                    text = &buffer[0];
                }

                cch = cch2;
            }
        }

        return cch;
    }

    // Context menu
    UINT SetCtxtMenuFlags(UINT uFlags)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetCtxtMenuFlags(this->m_hWnd, uFlags);
    }

    UINT GetCtxtMenuFlags() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetCtxtMenuFlags(this->m_hWnd);
    }

    // Searching and selection
    BOOL SetIgnoreCase(BOOL bIgnore)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetIgnoreCase(this->m_hWnd, bIgnore);
    }

    BOOL GetIgnoreCase() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetIgnoreCase(this->m_hWnd);
    }

    int FindItem(FindInfo &find) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_FindItem(this->m_hWnd, &find);
    }

    BOOL EnsureVisible(int nItem, int nCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_EnsureVisible(this->m_hWnd, nItem, nCol);
    }

    void SetTypeSelString(LPCTSTR lpszTypeSel)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        MListBox_SetTypeSelString(this->m_hWnd, lpszTypeSel);
    }
    template <ss_typename_param_k S>
    void SetTypeSelString(S const &lpszTypeSel)
    {
        this->SetTypeSelString(::stlsoft::c_str_ptr(lpszTypeSel));
    }

    LPCTSTR GetTypeSelString() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetTypeSelString(this->m_hWnd);
    }

    DWORD SetTypeSelTimeout(DWORD dwTimeout)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetTypeSelTimeout(this->m_hWnd, dwTimeout);
    }

    DWORD GetTypeSelTimeout() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetTypeSelTimeout(this->m_hWnd);
    }

    BOOL SetSelectImage(BOOL bSelectImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetSelectImage(this->m_hWnd, bSelectImage);
    }

    BOOL GetSelectImage() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetSelectImage(this->m_hWnd);
    }

    BOOL ShowImages(BOOL bShow)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_ShowImages(this->m_hWnd, bShow);
    }

    BOOL SetAlwaysSendDelete(BOOL bAlways)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetAlwaysSendDelete(this->m_hWnd, bAlways);
    }

    BOOL GetAlwaysSendDelete() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetAlwaysSendDelete(this->m_hWnd);
    }

    // File operations
    int Dir(UINT attr, LPCTSTR lpszWildCard)
    {
        return (int)SendMessage(LB_DIR, (WPARAM)attr, (LPARAM)lpszWildCard);
    }
    template <ss_typename_param_k S>
    int Dir(UINT attr, S const &lpszWildCard)
    {
        return this->Dir(attr, ::stlsoft::c_str_ptr(lpszWildCard));
    }

# if WINVER >= 0x0400
    int AddFile(LPCTSTR lpszFile)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_AddFile(this->m_hWnd, lpszFile);
    }
    template <ss_typename_param_k S>
    int AddFile(S const &lpszFile)
    {
        return this->AddFile(::stlsoft::c_str_ptr(lpszFile));
    }
# endif /* WINVER >= 0x0400 */

    UINT SetDirStyle(UINT uDirStyle)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_SetDirStyle(this->m_hWnd, uDirStyle);
    }

    UINT GetDirStyle() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MListBox_GetDirStyle(this->m_hWnd);
    }

#if 0
    BOOL DlgDirSelect(HWND hwnd, LPCTSTR lpString, int nIDListBox)
    {
        return DlgDirSelect(hwnd, (LPTSTR)lpString, __SYNSOFT_VAL_PATH_MAX, nIDListBox);
    }
#endif /* 0 */
};

typedef CSyListBoxT<CWindow>        CSyListBox;

/* /////////////////////////////////////////////////////////////////////////////
 * CSyHexViewT
 */

/// Class facade for the Synesis Software HexView Custom Control
template <class TBase>
class CSyHexViewT
    : public TBase
    , public SynesisControlCommonFeatures<CSyHexViewT<TBase> >
{
public:
    typedef TBase                   parent_class_type;
    typedef CSyHexViewT<TBase>      class_type;

/// \name Construction
/// @{
public:
    CSyHexViewT(HWND hWnd = NULL)
        : parent_class_type(hWnd)
    {}

    class_type &operator =(HWND hWnd)
    {
        m_hWnd = hWnd;
        return *this;
    }
/// @}

/// \name Attributes
/// @{
public:
    static LPCTSTR GetWndClassName()
    {
        return _T("SyHexView32");
    }
/// @}

/// \name HexView Operations
/// @{
public:
    /// Indicates the ownership of the data in the view
    enum Ownership
    {
            view    =   0                   //!< The control views the data. It does not own it. It assumes the pointer remains valid until SetData() called again or destroyed
        ,   copy    =   MHVDF_TAKECOPY      //!< The control takes a copy of the data, which it then uses. The copy becomes read-only
        ,   own     =   MHVDF_TAKEOWNERSHIP //!< The control takes ownership of the data, which it does not destroy until SetData() called again or destroyed. Client code may write to the data, but may not destroy / reallocate / resize it
    };

    BOOL    SetData(LPCHexViewData data)
    {
        return (BOOL)SyHexView_SetData(this->m_hWnd, data);
    }
    BOOL    SetData(void *pv, size_t cbData, UINT flags)
    {
        ATLASSERT(0 == (flags & ~(MHVDF_OWNERSHIP_MASK | MHVDF_DATA_MASK)));

        HexViewData data;

        data.flags      =   flags;
        data.pv         =   pv;
        data.cb         =   cbData;
        data.reserved   =   NULL;

        return this->SetData(&data);
    }

    BOOL    SetData(void *pv, size_t cbData)
    {
        return this->SetData(pv, cbData, MHVDF_VIEW | MHVDF_DATA_UNSPECIFIED);
    }

    BOOL    SetHGLOBAL(HGLOBAL hMem, size_t cbMem, Ownership = view)
    {
        ATLASSERT(0 == (view & ~MHVDF_OWNERSHIP_MASK));

        if(0 == cbMem)
        {
            cbMem = ::GlobalSize(hMem);
        }

        return this->SetData(hMem, cbMem, MHVDF_DATA_HGLOBAL | view);
    }

    BOOL    SetFileMapping(HANDLE hFileMap, size_t cbFileMap, Ownership = view)
    {
        ATLASSERT(0 == (view & ~MHVDF_OWNERSHIP_MASK));

        return this->SetData(hFileMap, cbFileMap, MHVDF_DATA_FILE_MAPPING | view);
    }

    BOOL    SetMappedView(void *pvMappedView, size_t cbFileMap, Ownership = view)
    {
        ATLASSERT(0 == (view & ~MHVDF_OWNERSHIP_MASK));

        return this->SetData(hFileMap, cbFileMap, MHVDF_DATA_FILE_MAPPING | view);
    }

    ///
    ///
    /// \return 0 on success, otherwise a Win32 error code indicating the reason for failure
    DWORD   SetFile(LPCTSTR fileName)
    {
        return SyHexView_SetFile(this->m_hWnd, fileName);
    }
    template <ss_typename_param_k S>
    DWORD   SetFile(S const &fileName)
    {
        return this->SetFile(::stlsoft::c_str_ptr(fileName));
    }

    BOOL    GetData(LPHexViewData data) const
    {
        return (BOOL)SyHexView_GetData(this->m_hWnd, data);
    }

    BOOL    SetBorders(HexViewBorders const &borders)
    {
        return (BOOL)SyHexView_SetBorders(this->m_hWnd, &borders);
    }
    BOOL    SetBorders( unsigned    vertical
                    ,   unsigned    horizontal
                    ,   unsigned    characters
                    ,   unsigned    rows
                    ,   unsigned    columns)
    {
        HexViewBorders  borders;

        borders.vertical    =   vertical;
        borders.horizontal  =   horizontal;
        borders.characters  =   characters;
        borders.rows        =   rows;
        borders.columns     =   columns;

        return this->SetBorders(&borders);
    }
    BOOL    GetBorders(HexViewBorders &borders) const
    {
        return (BOOL)SyHexView_GetBorders(this->m_hWnd, &borders);
    }

    LONG    CopyToClipboard(UINT flags) const
    {
        return (LONG)SyHexView_CopyToClipboard(this->m_hWnd, flags, NULL);
    }

    LONG    CopyToClipboard(UINT flags, LPCSTR prefix) const
    {
        return (LONG)SyHexView_CopyToClipboard(this->m_hWnd, flags, prefix);
    }
/// @}

/// \name Context menu
/// @{
public:
    UINT SetCtxtMenuFlags(UINT uFlags)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_SetCtxtMenuFlags(this->m_hWnd,uFlags);
    }

    UINT GetCtxtMenuFlags() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_GetCtxtMenuFlags(this->m_hWnd);
    }

    // Searching and selection
    BOOL SetIgnoreCase(BOOL bIgnore)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_SetIgnoreCase(this->m_hWnd,bIgnore);
    }

    BOOL GetIgnoreCase() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_GetIgnoreCase(this->m_hWnd);
    }

    int FindItem(FindInfo &find) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_FindItem(this->m_hWnd,&find);
    }

    BOOL EnsureVisible(int nItem, int nCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_EnsureVisible(this->m_hWnd,nItem, nCol);
    }

    BOOL SetSelectImage(BOOL bSelectImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_SetSelectImage(this->m_hWnd,bSelectImage);
    }

    BOOL GetSelectImage() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_GetSelectImage(this->m_hWnd);
    }

    BOOL SetAlwaysSendDelete(BOOL bAlways)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_SetAlwaysSendDelete(this->m_hWnd,bAlways);
    }

    BOOL GetAlwaysSendDelete() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MHexView_GetAlwaysSendDelete(this->m_hWnd);
    }
/// @}
};

typedef CSyHexViewT<CWindow>        CSyHexView;

/* /////////////////////////////////////////////////////////////////////////////
 * CSyEditBarT
 */

/// Class facade for the Synesis Software EditBar Custom Control
template <class TBase>
class CSyEditBarT
    : public TBase
    , public SynesisControlCommonFeatures<CSyEditBarT<TBase> >
{
private:
    typedef ::stlsoft::auto_buffer<TCHAR, ::stlsoft::malloc_allocator<TCHAR>, 256>  buffer_type;
public:
    typedef TBase                                                                   parent_class_type;
    typedef CSyEditBarT<TBase>                                                      class_type;

/// \name Construction
/// @{
public:
    CSyEditBarT(HWND hWnd = NULL)
        : parent_class_type(hWnd)
    {}

    class_type &operator =(HWND hWnd)
    {
        m_hWnd = hWnd;
        return *this;
    }
/// @}

/// \name Attributes
/// @{
public:
    static LPCTSTR GetWndClassName()
    {
        return _T("SyEditBar32");
    }
/// @}

/// \name EditBar Operations
/// @{
public:
    UINT    SetBtnState(UINT idBtn, UINT uState)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetBtnState(this->m_hWnd, idBtn, uState);
    }
    UINT    GetBtnState(UINT idBtn) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetBtnState(this->m_hWnd, idBtn);
    }

    DWORD   SetItemData(UINT idBtn, DWORD dw)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetItemData(this->m_hWnd, idBtn, dw);
    }
    DWORD   GetItemData(UINT idBtn) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetItemData(this->m_hWnd, idBtn);
    }

    BOOL    SetAccelerator(UINT idBtn, const ACCEL FAR *pAccel)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetAccelerator(this->m_hWnd, idBtn, pAccel);
    }
    BOOL    GetAccelerator(UINT idBtn, ACCEL FAR *pAccel)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetAccelerator(this->m_hWnd, idBtn, pAccel);
    }

    BOOL    SetSticky(BOOL bSticky)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetSticky(this->m_hWnd, bSticky);
    }
    BOOL    GetSticky() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetSticky(this->m_hWnd);
    }

    HIMAGELIST  SetItemImageList(HIMAGELIST hil, UINT uBtns)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetItemImageList(hwnd, hil, uBtns);
    }
    HIMAGELIST  GetItemImageList()
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetItemImageList(hwnd);
    }

    HIMAGELIST  SetHighlightImageList(HIMAGELIST hil)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetHighlightImageList(this->m_hWnd, hil);
    }
    HIMAGELIST  GetHighlightImageList() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetHighlightImageList(this->m_hWnd);
    }

/* Tooltips. */
#if WINVER >= 0x0400
    BOOL    EnableTooltips(BOOL bEnable)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_EnableTooltips(this->m_hWnd, bEnable);
    }
    BOOL    EnableTooltips() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_AreTooltipsEnabled(this->m_hWnd);
    }

    BOOL    SetTooltipText(UINT idBtn, LPCTSTR text)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetTooltipText(this->m_hWnd, idBtn, text);
    }
    template <ss_typename_param_k S>
    BOOL    SetTooltipText(UINT idBtn, S const &text)
    {
        return this->SetTooltipText(idBtn, ::stlsoft::c_str_ptr(text));
    }
    int     GetTooltipText(UINT idBtn, LPTSTR text) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetTooltipText(this->m_hWnd, idBtn, text);
    }
    int     GetTooltipTextLength(UINT idBtn) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetTooltipTextLen(this->m_hWnd, idBtn);
    }
    template <ss_typename_param_k S>
    int GetTooltipText(UINT idBtn, S &text) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        int cch =   this->GetTooltipTextLen(idBtn);

        if(cch >= 0)
        {
            buffer_type buffer(1 + cch);

            if(buffer.empty())
            {
                cch = -1;
            }
            else
            {
                int cch2    =   this->GetTooltipText(idBtn, &buffer[0]);

                if(cch2 >= 0)
                {
                    text = &buffer[0];
                }

                cch = cch2;
            }
        }

        return cch;
    }

#endif /* WINVER >= 0x0400 */

    HWND    SetAssociate(HWND hwndAssoc)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetAssociate(this->m_hWnd, hwndAssoc);
    }
    HWND    GetAssociate() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetAssociate(this->m_hWnd);
    }

    BOOL    SetNotifyAssociate(BOOL bNotifyAssoc)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetNotifyAssociate(this->m_hWnd, bNotifyAssoc);
    }
    BOOL    GetNotifyAssociate() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetNotifyAssociate(this->m_hWnd);
    }

    void    InvalidateButtons()
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        MEditBar_InvalidateButtons(this->m_hWnd);
    }

    BOOL    GetRect(UINT uRect, RECT &rc)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetRect(this->m_hWnd, uRect, &rc);
    }

/// @}

/// \name Context menu
/// @{
public:
    UINT SetCtxtMenuFlags(UINT uFlags)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetCtxtMenuFlags(this->m_hWnd,uFlags);
    }

    UINT GetCtxtMenuFlags() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetCtxtMenuFlags(this->m_hWnd);
    }

    // Searching and selection
    BOOL SetIgnoreCase(BOOL bIgnore)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetIgnoreCase(this->m_hWnd,bIgnore);
    }

    BOOL GetIgnoreCase() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetIgnoreCase(this->m_hWnd);
    }

    int FindItem(FindInfo &find) const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_FindItem(this->m_hWnd,&find);
    }

    BOOL EnsureVisible(int nItem, int nCol)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_EnsureVisible(this->m_hWnd,nItem, nCol);
    }

    BOOL SetSelectImage(BOOL bSelectImage)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetSelectImage(this->m_hWnd,bSelectImage);
    }

    BOOL GetSelectImage() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetSelectImage(this->m_hWnd);
    }

    BOOL SetAlwaysSendDelete(BOOL bAlways)
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_SetAlwaysSendDelete(this->m_hWnd,bAlways);
    }

    BOOL GetAlwaysSendDelete() const
    {
        ATLASSERT(::IsWindow(this->m_hWnd));

        return MEditBar_GetAlwaysSendDelete(this->m_hWnd);
    }
/// @}
};

typedef CSyEditBarT<CWindow>        CSyEditBar;

/* ////////////////////////////////////////////////////////////////////////// */


/// Function object used to insert items at the back of list-views
// [[synesis:class:function-class:unary-function: sylistbox_back_inserter]]
struct sylistbox_back_inserter
    : public stlsoft::unary_function_output_iterator_adaptor<sylistbox_back_inserter>
{
    typedef char                    ws_char_a_t;
    typedef wchar_t                 ws_char_w_t;
    typedef sylistbox_back_inserter class_type;

    enum
    {
        image_none  =   MCCIV_NO_IMAGE
    };

public:
    /// Construct with the target list-view window
    ss_explicit_k sylistbox_back_inserter(HWND hwndListBox, int iImage = image_none)
        : m_hwndListBox(hwndListBox)
        , m_iImage(iImage)
    {}

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    /// Function call operator taking the item string
    template <ss_typename_param_k S>
    void operator ()(S &s)
    {
        insert_item(c_str_ptr(s));
    }
#else /* ? __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */
    /// Function call operator taking the item string
    void operator ()(ws_char_a_t const *s)
    {
        insert_item(s);
    }
    /// Function call operator taking the item string
    void operator ()(ws_char_w_t const *s)
    {
        insert_item(s);
    }
#endif // __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT

/// \name Iterator functions
public:
#if 0
    class_type &operator ++()
    {
        return *this;
    }

    class_type operator ++(int)
    {
        return *this;
    }

    class_type &operator *() const
    {
        return *const_cast<class_type*>(this);
    }

#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    /// Function call operator taking the item string
    template <ss_typename_param_k S>
    void operator =(S const &s)
    {
        insert_item(c_str_ptr(s));
    }
#else /* ? __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */
    void operator =(ws_char_a_t const *s)
    {
        insert_item(s);
    }
    void operator =(ws_char_w_t const *s)
    {
        insert_item(s);
    }
#endif // __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
#endif /* 0 */

private:
    void insert_item(ws_char_a_t const *s)
    {
        CellItemA   item;

        item.mask       =   MCCIF_TYPE ;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<ws_char_a_t *>(s);
        item.nItem      =   -1;
        item.nSubItem   =   0;

        if(image_none != m_iImage)
        {
            item.mask |= MCCIF_IMAGE;
            item.image = m_iImage;
        }

        ::SendMessage(m_hwndListBox, MLBM_INSERTITEM, 0, (LPARAM)(CellItem*)&item);
    }

    void insert_item(ws_char_w_t const *s)
    {
        CellItemW   item;

        item.mask       =   MCCIF_TYPE ;
        item.type       =   MCCIT_STRING;
        item.pszVal     =   const_cast<ws_char_w_t *>(s);
        item.nItem      =   -1;
        item.nSubItem   =   0;

        if(image_none != m_iImage)
        {
            item.mask |= MCCIF_IMAGE;
            item.image = m_iImage;
        }

        ::SendMessage(m_hwndListBox, MLBM_INSERTITEM, 0, (LPARAM)(CellItem*)&item);
    }

private:
    HWND    m_hwndListBox;
    int     m_iImage;

private:
    void operator =(class_type const &);
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WTLSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
} // namespace wtlstl
# else
} // namespace wtlstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WTLSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !WTLSTL_INCL_H_WTLSTL_X_SYNESIS_CONTROLS */

/* ////////////////////////////////////////////////////////////////////////// */
